#include <stdio.h>
#include "objfile.h"

#if defined(__APPLE__)
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#include "mesh.h"

Mesh::Mesh()
{
	vertices = normals = 0;
	vcount = 0;
}

Mesh::~Mesh()
{
	delete [] vertices;
	delete [] normals;
}

void Mesh::set_position(float x, float y, float z)
{
	this->pos = Vector3(x, y, z);
}

bool Mesh::load(const char *fname)
{
	objfile *objf = objf_load(fname);
	if(!objf) {
		fprintf(stderr, "failed to load obj file: %s\n", fname);
		return false;
	}

	vcount = objf_vertex_count(objf);

	vertices = new Vector3[vcount];
	normals = new Vector3[vcount];

	for(int i=0; i<vcount; i++) {
		float *v = objf_vertex(objf, i);
		vertices[i].x = v[0];
		vertices[i].y = v[1];
		vertices[i].z = v[2];

		float *n = objf_normal(objf, i);
		normals[i].x = n[0];
		normals[i].y = n[1];
		normals[i].z = n[2];
	}

	objf_free(objf);
	return true;
}

void Mesh::draw() const
{
	glPushMatrix();
	glTranslatef(pos.x, pos.y, pos.z);

	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_NORMAL_ARRAY);

	glVertexPointer(3, GL_FLOAT, 0, (float*)vertices);
	glNormalPointer(GL_FLOAT, 0, (float*)normals);

	glDrawArrays(GL_TRIANGLES, 0, vcount);

	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_NORMAL_ARRAY);

	glPopMatrix();
}

void Mesh::draw_bsph() const
{
	// TODO
}

bool Mesh::intersect(const Ray &ray, HitPoint *pt) const
{
	return false;	// TODO
}
